@@ -580,3 +580,14 @@ test_that("testing family specification", {
                  "of mode.*not found")
 
 })
+
+## GH #1213
+test_that("error message for ziGamma with zeros",
+{
+  dd <- data.frame(y = c(1,0))
+  expect_error(glmmTMB(y ~ 1, data = dd, family = ziGamma(link = "log")),
+               "unless `ziformula`")
+  suppressWarnings(g <- glmmTMB(y ~ 1, data = dd, family = ziGamma(link = "log"),
+          ziformula = ~1))
+  expect_is(g, "glmmTMB")
+})
